var pg_port=1212;
var pg_host="my-website.com:9082/BLUDB";
var pg_user="root";
var pg_pass="sup3rstr0ngpass1ForPG";

var mongo_uri = "mongodb+srv://testuser:hub24aoeu@checkov-is-awesome-gg273.mongodb.net/test?retryWrites=true&w=majority";

// This should pass
var password = process.env.PASSWORD

