import { aws_glue as glue } from 'aws-cdk-lib';

const cfnSecurityConfigurationProps1:  glue.CfnCrawlerProps = {
  name: 'name',
};

const cfnSecurityConfigurationProps2: glue.CfnDevEndpointProps = {
  name: 'name',
};

const cfnSecurityConfigurationProps3: glue.CfnJobProps = {
  name: 'name',
};
